SUBROUTINE test_Gauss(n,z1,alpha1,ad,cvm,reject_a,reject_c)
!******************************************************************************
! FORTRAN 90 subroutine
! File: gauss.f90
! Coded by: Nusrat Jahan
!
! INPUT:
! n        = a scalar containing the length of the estimated normalized bispectrum.
! z1       = a double precision array of length n containing the square modulus 
!            of the normalized estimated bispectrum.
! alpha1   = a double precision scalar containing the level of significance
!            of the test.
!
! ad       = a double precision scalar containing the value of Anderson-Darling (AD)
!            test statistic.
! cvm      = a double precision scalar containing the value of Cramer-von Mises (CvM)
!            test statistic.
!
! OUTPUT:
! reject_a = number of rejections AD test statistic
! reject_c = number of rejections CvM test statistic
!
! Reference:
! Jahan, N. and Harvill, J.L. (2008).
!   Bispectral-based goodness-of-fit tests of Gaussianity
!   and linearity of stationary time series.
!   Communications in Statistics: Theory and Methods, 37(20), 3216--3227.
!   DOI: 10.1080/03610920802133319.
!*****************************************************************************
!
  USE define_types
  IMPLICIT NONE
  INTEGER (i4b), INTENT(in):: n
  INTEGER (i4b), INTENT(inout):: reject_a,reject_c
  REAL(dp), INTENT(inout) :: alpha1,ad,cvm
  REAL(dp), INTENT(inout), DIMENSION(n):: z1
!
  INTEGER (i4b):: i,j,k
  REAL(dp), DIMENSION(n)::  z2, q
  REAL(dp) :: a, cvm1, cvm2, cv, rr, del, adiff
!
  call qksort(n,z1,z2)
!
  q = 1.0_dp - EXP(-(z2/2.0_dp))
!
!****************************************************************************
! Hinich's test statistic for testing time series Gaussianity.
!****************************************************************************
!
  j  = n/4
  k  = n*(3/4)
  rr = q(k) - q(j)

!*****************************************************************************
!
! Cramer vom Mises EDF test statistic
!******************************************************************************!
  cvm1 =0.0_dp
  DO i = 1, n
     cvm1 = cvm1 + (q(i) - (2.0_dp*DBLE(i)-1.0_dp)/(2*n))**2
  END DO
!
  cvm2 = cvm1 + 1.0_dp/(12*n)
!
  cvm = (cvm2 - .4_dp/DBLE(n) +.6_dp/DBLE(n**2))*(1.0_dp + 1.0_dp/DBLE(n))
!
!
  a = 0.0_dp
  DO i = 1, n
     a = a + (2.0_dp*DBLE(i) - 1.0_dp)*LOG(q(i)) + (2.0_dp*n + 1 - 2.0_dp*DBLE(i))* LOG(1-q(i))
  END DO
!
  ad = -n -(1/DBLE(n))*a
!
!
  del   = 0.00000001_dp
  adiff = abs(alpha1 - 0.25_dp)
  cv    = 2.492_dp
!
  IF(adiff .LE. del) THEN
     cv = 1.248_dp
  ELSE 
     adiff = ABS(alpha1 - 0.15_dp)
  END IF
  IF(adiff .EQ. del) THEN
     cv = 1.610_dp
  ELSE 
     adiff = ABS(alpha1 - 0.10_dp)
  END IF

  IF(adiff .EQ. del) THEN 
     cv = 1.933_dp
  ELSE 
      adiff = ABS(alpha1 - 0.025_dp)
  END IF
  IF(adiff .EQ. del) THEN
     cv = 3.070_dp
  ELSE 
     adiff = ABS(alpha1 - 0.01_dp)
  END IF
  IF(adiff .EQ. del) THEN
     cv = 3.857_dp
  ELSE 
     adiff = ABS(alpha1 - 0.005_dp)
  END IF
  IF(adiff .EQ. del) THEN
     cv = 4.500_dp
  ELSE 
     adiff = ABS(alpha1 - 0.001_dp)
  END IF
  IF(adiff .EQ. del) THEN
     cv = 6.000_dp
  ELSE
     cv = 2.492_dp
  END IF
!
  IF(ad .GT. cv) THEN 
     reject_a = 1
  ELSE IF (ad .LE. cv) THEN
     reject_a = 0
  END IF
!
  IF(cvm .GT. .461) THEN 
     reject_c = 1
  ELSE IF (cvm .LE. .461) THEN
     reject_c = 0
  END IF
!
  RETURN
!
END SUBROUTINE test_Gauss
